@@ -1,10 +1,13 @@
 #' Univariate analysis of randomized response data
 #' 
 #' Analyse a data vector \code{response} with a specified RR model (e.g., \code{Warner}) with known randomization probability \code{p}
-#'  @param response either vector of responses containing 0='no' and 1='yes' or name of response variable in \code{data}. In Kuk's card playing method (\code{Kuk}), the observed response variable gives the number of red cards. For the Forced Response (\code{FR}) model, response values are integers from 0 to (m-1), where 'm' is the number of response categories. 
-#'  @param data optional \code{data.frame} containing the response variable
-#'  @param model defines RR model. Available models: \code{"Warner"}, \code{"UQTknown"}, \code{"UQTunknown"}, \code{"Mangat"}, \code{"Kuk"},\code{"FR"}, \code{"Crosswise"}, \code{"CDM"}, \code{"CDMsym"}, \code{"SLD"}, \code{"mix.norm"}, \code{"mix.exp"},\code{"mix.unknown"}, or \code{"custom"}. See argument \code{p} or type \code{vignette('RRreg')} for detailed specifications.
-#'  @param p randomization probability defined as a single probability for
+#' @param response either vector of responses containing 0='no' and 1='yes' or name of response variable in \code{data}. In Kuk's card playing method (\code{Kuk}), the observed response variable gives the number of red cards. For the Forced Response (\code{FR}) model, response values are integers from 0 to (m-1), where 'm' is the number of response categories. 
+#' @param data optional \code{data.frame} containing the response variable
+#' @param model defines RR model. Available models: \code{"Warner"}, \code{"UQTknown"}, \code{"UQTunknown"}, \code{"Mangat"}, \code{"Kuk"},\code{"FR"}, \code{"Crosswise"}, \code{"CDM"}, \code{"CDMsym"}, \code{"SLD"}, \code{"mix.norm"}, \code{"mix.exp"},\code{"mix.unknown"}, or \code{"custom"}. See argument \code{p} or type \code{vignette('RRreg')} for detailed specifications.
+#' @param p randomization probability (see details or \code{vignette("RRreg")})
+#' @param group a group vector of the same length as \code{response} containing values 1 or 2, only required for two-group models, which specify different randomization probabilities for two groups, e.g., \code{CDM} or \code{SLD}. If a data.frame \code{data} is provided, the variable \code{group} is searched within it.
+#' @param MLest whether to use \code{optim} to get ML instead of moment estimates (only relevant if pi is outside of [0,1])
+#' @details Each RR design \code{model} differs in the definition of the randomization probability \code{p}, which is defined as a single probability for
 #'  \itemize{
 #'  \item \code{"Warner"}: Probabiltiy to get sensitive Question 
 #'  \item \code{"Mangat"}: Prob. for noncarriers to respond truthfully (i.e., with No=0)
@@ -27,8 +30,6 @@
 #'  \item \code{"mix.exp"}: 2-valued vector - Prob. to respond to sensitive question and mean of the masking exponential distribution of the unrelated question
 #'  \item \code{"mix.unknown"}: 2-valued vector - Prob. of responding to sensitive question in group 1 and 2, respectively
 #'  }
-#'  @param group a group vector of the same length as \code{response} containing values 1 or 2, only required for two-group models, which specify different randomization probabilities for two groups, e.g., \code{CDM} or \code{SLD}. If a data.frame \code{data} is provided, the variable \code{group} is searched within it.
-#' @param MLest whether to use \code{optim} to get ML instead of moment estimates (only relevant if pi is outside of [0,1])
 #' @return an \code{RRuni} object, can by analyzed by using \code{\link{summary}}
 #' @seealso \code{vignette('RRreg')} or \url{https://dl.dropboxusercontent.com/u/21456540/RRreg/index.html} for a detailed description of the RR models and the appropriate definition of \code{p} 
 #' @examples 
@@ -51,56 +52,56 @@ RRuni <- function(response, data, model, p, group = NULL, MLest=TRUE){
   # extract column 'response' from data.frame 'data'
   if ( !missing(data)){
     try( {data <- as.data.frame(data)
-          response <-  eval(substitute(response),data, parent.frame())
-          })
+    response <-  eval(substitute(response),data, parent.frame())
+    })
   }
   n <- length(response)
   model <- match.arg(model, modelnames())
   if ( is2group(model) &&  !missing(data) ){
-      try({data <- as.data.frame(data)
-         group <-  eval(substitute(group),data, parent.frame())
-           },silent=T)
+    try({data <- as.data.frame(data)
+    group <-  eval(substitute(group),data, parent.frame())
+    },silent=T)
   }
   if(!is2group(model))
     group <- rep(1, n)
   RRcheck.xpgroup(model,response,p,group,"response")
   
   res <- switch(model,
-         "Warner" = RRuni.Warner(response,p),
-         "UQTknown" = RRuni.UQTknown(response,p),
-         "Mangat" = RRuni.Mangat(response,p),
-         "FR" = RRuni.FR(response,p),
-         "custom" = RRuni.custom(response, p, group),
-         
-         "Kuk" = RRuni.Kuk(response,p),
-         "UQTunknown" = RRuni.UQTunknown(response,p,group),
-         "Crosswise" = RRuni.Crosswise(response,p),
-         "SLD" = RRuni.SLD(response,p,group),
-         "CDM" = RRuni.CDM(response,p,group),
-         "CDMsym" = RRuni.CDMsym(response,p,group),
-         
-         "mix.norm" = RRuni.mix.norm(response,p),
-         "mix.exp" = RRuni.mix.exp(response,p),
-         "mix.unknown" = RRuni.mix.unknown(response,p, group)
-         )
+                "Warner" = RRuni.Warner(response,p),
+                "UQTknown" = RRuni.UQTknown(response,p),
+                "Mangat" = RRuni.Mangat(response,p),
+                "FR" = RRuni.FR(response,p),
+                "custom" = RRuni.custom(response, p, group),
+                
+                "Kuk" = RRuni.Kuk(response,p),
+                "UQTunknown" = RRuni.UQTunknown(response,p,group),
+                "Crosswise" = RRuni.Crosswise(response,p),
+                "SLD" = RRuni.SLD(response,p,group),
+                "CDM" = RRuni.CDM(response,p,group),
+                "CDMsym" = RRuni.CDMsym(response,p,group),
+                
+                "mix.norm" = RRuni.mix.norm(response,p),
+                "mix.exp" = RRuni.mix.exp(response,p),
+                "mix.unknown" = RRuni.mix.unknown(response,p, group)
+  )
   # print(c(res$pi, res$piSE)) 
-  estNotML <- any(res$pi<0)
+  estNotML <- any(res$pi <= 0 | res$pi >= 1)
   if(is2group(model) && model != "mix.unknown"){
     estNotML <- estNotML || switch(model,
-                                  "SLD" = res$t<0 | res$t>1,
-                                  "UQTunknown" = res$piUQ<0 | res$piUQ>1,
-                                  "CDM" = res$gamma<0 | res$gamma>1,
-                                  "CDMsym" = res$gamma<0 | res$gamma>1)
+                                   "SLD" = res$t <= 0 | res$t >= 1,
+                                   "UQTunknown" = res$piUQ <= 0 | res$piUQ >= 1,
+                                   "CDM" = res$gamma <= 0 | res$gamma >= 1,
+                                   "CDMsym" = res$gamma <= 0 | res$gamma >= 1)
   }
   if (MLest  && estNotML && !(model %in% c("mix.norm","mix.exp", "mix.unknown")) ){ 
     # starting values
-    start <- RRcheck.param(res$pi)
+    start <- min(max(RRcheck.param(res$pi),1e-4),1-1e-4)
     # adjust for categorical responses:
     ncat <- ifelse(length(res$pi) <=2, 2, length(res$pi))
-    if(length(res$pi)>1)
+    if(length(res$pi) > 1 && model != "custom")
       start <- start[-1]
     if(sum(start) >1)
-      start <- start/sum(start)- .0001
+      start <- start/sum(start)
     
     if(is2group(model))
       start <- c(start, runif(1))
@@ -188,13 +189,13 @@ summary.RRuni<-function(object,...){
               cbind(object$gamma,object$gammaSE,zval_g,pnorm(zval_g,lower.tail=F)))
     rownames(TAB)=c("pi","beta","gamma")
   }
-#   if (object$model %in% c("CDMsym")){
-#     zval_g=object$gamma/object$gammaSE
-#     TAB=rbind(TAB,
-# #               cbind(object$beta,object$betaSE,zval_b,pnorm(-abs(zval_b))),
-#               cbind(object$gamma,object$gammaSE,zval_g,pnorm(zval_g,lower.tail=F)))
-#     rownames(TAB)=c("pi","gamma")
-#   }
+  #   if (object$model %in% c("CDMsym")){
+  #     zval_g=object$gamma/object$gammaSE
+  #     TAB=rbind(TAB,
+  # #               cbind(object$beta,object$betaSE,zval_b,pnorm(-abs(zval_b))),
+  #               cbind(object$gamma,object$gammaSE,zval_g,pnorm(zval_g,lower.tail=F)))
+  #     rownames(TAB)=c("pi","gamma")
+  #   }
   if (object$model %in% c("mix.unknown")){
     zval_g=object$piUQ/object$piUQSE
     TAB=rbind(TAB,
@@ -219,7 +220,7 @@ print.summary.RRuni<-function(x,...){
   printCoefmat( round(x$coefficients,6))
   if (x$model=="SLD"){
     cat("\n(for the parameter t, i.e. probability of true responding of carriers,
-        the test is H0: t=1; H1: t<1 and the one-sided probability value is given)")
+ the test is H0: t=1; H1: t<1 and the one-sided probability value is given)")
   }
 }
 
